#!/usr/bin/env python

# SECLAF Sequence Classification Framework
# Copyright (c) 2016-2017 Balazs Szalkai

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This script produces the class hierarchy ("tree") file for Gene Ontology.
# Input: go.obo (created by get_uniprot_go.sh)
# Output: go.tre, go.alt_id, go.obsolete

import os,re,math,time,sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
from seclaf import *

dir = os.path.dirname(__file__)+"/"
fn = dir+"go.obo"
fnOut = dir+"go.tre"
fnAltId = dir+"go.alt_id"
fnObsolete = dir+"go.obsolete"
alt_id_dict = {}
obsolete = {}
t = Tree()
with open(fn) as f:
	for line in f:
		line = line.rstrip()
		if line.startswith("[Term]"):
			id = None
		elif line.startswith("id: "):
			id = line.split(" ")[1]
			if id.startswith("GO:"):
				t.addNode(id)
		elif line.startswith("name: "):
			if id.startswith("GO:") and id not in obsolete:
				name = line[6:]
				t.nodeDesc[id] = name
		elif line.startswith("is_a: "):
			if id.startswith("GO:") and id not in obsolete:
				parent = line.split(" ")[1]
				if parent.startswith("GO:") and parent not in obsolete:
					t.addParent(id, parent) 
		elif line.startswith("alt_id: "):
			if id.startswith("GO:") and id not in obsolete:
				alt_id = line.split(" ")[1]
				if alt_id.startswith("GO:") and alt_id not in obsolete:
					alt_id_dict[alt_id] = id
		elif line.startswith("is_obsolete: true"):
			if id.startswith("GO:") and id not in obsolete:
				t.removeNode(id)
				obsolete[id] = True
				
t.save(fnOut)
print(fnOut+" successfully saved")
with open(fnAltId, "w") as fAltId:
	for k, v in diter(alt_id_dict):
		fAltId.write(k+"\t"+v+"\n")
print(fnAltId+" successfully saved")
with open(fnObsolete, "w") as fObsolete:
	for n in obsolete:
		fObsolete.write(n+"\n")
print(fnObsolete+" successfully saved")
