#!/usr/bin/env python

# SECLAF Sequence Classification Framework
# Copyright (c) 2016-2017 Balazs Szalkai

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This script makes the sequence annotation file for training or testing the network.
# Input: go.tre, go.alt_id (created by make_go_tree.py), uniprot_go.tab (created by get_uniprot_go.sh)
# Output: uniprot_go.ann

import os,re,math,time,sys,random

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))	
from seclaf import *

tree = Tree()
tree.load("go.tre")

alt_id = {}
with open("go.alt_id") as f:
	for line in f:
		tok = line.rstrip().split("\t")
		if len(tok) == 2:
			alt_id[tok[0]] = tok[1]

dir = os.path.dirname(__file__)+"/"
fn = dir+"uniprot_go.tab"
annList = []
with open(fn) as f:
	tok = f.readline().rstrip().split("\t")
	id_go = tok.index("Gene ontology IDs")
	id_seq = tok.index("Sequence")
	for line in f:
		tok = line.rstrip().split("\t")
		if len(tok) < max(id_go, id_seq)+1:
			print(line)
			continue
		seq = tok[id_seq]
		classif = tok[id_go].replace(";", "&").replace(",", "&").replace(" ", "").split("&")
		for i in range(len(classif)):
			if classif[i] in alt_id:
				classif[i] = alt_id[classif[i]]
		classif = [c for c in classif if c in tree.parents] # filter by tree nodes
		if len(classif) == 0:
			print("Unclassified sequence %s..." % seq[:25])
		else:
			annList.append((seq, classif))
		
random.shuffle(annList)

fnAnn = dir+"uniprot_go.ann"
ann = AnnotationWriter(fnAnn)
for seq_classif in annList:
	ann.write(seq_classif[0], seq_classif[1])
ann.close()
print(fnAnn+" successfully saved")
