#!/usr/bin/env python

# SECLAF Sequence Classification Framework
# Copyright (c) 2016-2017 Balazs Szalkai

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This script makes the UniProt family tree along with the sequence annotation file for training or testing the network.
# Input: uniprot-all.tab (created by get_uniprot_fam.sh)
# Output: uniprot_fam.tre, uniprot_fam.ann

import os,re,math,time,sys,random

sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))	
from seclaf import *

# returns: family node with all parent nodes (list, the family node is the first entry)
# if multiple families are present, separated with ;, then returns all the nodes
def decodeFamilyNodes(protein_families):
	protein_families = protein_families.strip()
	assert protein_families.endswith("family"), protein_families
	
	if protein_families.find(";") >= 0:
		# split them, multiple entries
		pf = protein_families.split(";")
		result = []
		for f in pf:
			result += decodeFamilyNodes(f.strip())
		return result
		
	if protein_families in descToNode:
		node = descToNode[protein_families]
		return t.includeParents(node)
	else:
		if protein_families.count("family") >= 2:
			parentfamily = protein_families[ : protein_families[:-1].rfind("family")+len("family")] # find the second last occurrence
			parentnodes = decodeFamilyNodes(parentfamily)
			nodeId = protein_families[len(parentfamily):]
		else:
			parentnodes = []
			nodeId = protein_families
			
		nodeId = "".join([c for c in nodeId.lower() if (c >= 'a' and c <= 'z') or (c >= '0' and c <= '9')])
		nodeId = nodeId[:16]
		nodeId = t.forceAddNode(nodeId)
		if len(parentnodes) >= 1:
			t.addParent(nodeId, parentnodes[0])
		t.nodeDesc[nodeId] = protein_families
		descToNode[protein_families] = nodeId
		
		return [nodeId] + parentnodes

def safe_unpack(a, length, defaultValue = ""):
	if len(a) > length:
		return a[:length]
	if len(a) == length:
		return a
	return list(a) + [defaultValue]*(length-len(a))
		
dir = os.path.dirname(__file__)+"/"
fnTab = dir+"uniprot-all.tab"
assert os.path.isfile(fnTab), fnTab + " does not exist, please run get_uniprot_fam.sh"
fnClass = dir+"uniprot_fam.ann"
fnTree = dir+"uniprot_fam.tre"
t = Tree()
descToNode = {}
nofamily = 0
allcount = 0
annList = []
with open(fnClass, "w") as fclass:
	with open(fnTab) as f:
		for line in f:
			line = line.rstrip()
			if len(line) < 5 or line.startswith("Entry"):
				continue
			allcount += 1
			tok = line.split('\t')
			entry, entry_name, protein_names, gene_names, organism, length, protein_families, seq, gene_ontology = safe_unpack(tok, 9)
			#assert seq.isupper()
			if protein_families == "":
				nofamily += 1
				#sys.stderr.write(entry+" ")
				continue
			nodes = decodeFamilyNodes(protein_families)
			annList.append((seq, nodes, entry))
t.save(fnTree)

random.shuffle(annList)

aw = AnnotationWriter(fnClass)
for x in annList:
	aw.write(*x)
aw.close()

print "All count: "+str(allcount)
print "With no family: "+str(nofamily)
